@@ -98,7 +98,7 @@ CausalKinetiX <- function(D,
     pars$include.vars <- NA
   }
   if(!exists("maineffect.models", pars)){
-    pars$maineffect.models <- TRUE
+    pars$maineffect.models <- FALSE
   }
   if(!exists("rm.target",pars)){
     pars$rm.target <- FALSE
@@ -125,6 +125,9 @@ CausalKinetiX <- function(D,
     if(pars$regression.class == "signed.OLS"){
       signed <- TRUE
     }
+    else{
+      signed <- FALSE
+    }
   }
   else{
     signed <- FALSE
